import org.opencv.core.*;
import org.opencv.videoio.VideoCapture;
import org.opencv.highgui.HighGui;
import org.opencv.imgproc.Imgproc;
import org.opencv.imgproc.Moments;
import org.opencv.videoio.Videoio;

import java.util.ArrayList;
import java.util.List;
import java.awt.Point;

public class Tracker implements Runnable {

    private final int CAM = 1;                  // Kamera-ID (0 für Webcam, 1 für andere Cams)
    private final int EXP = 0;                  // Belichtungszeit (idealerweise: 0)
    private final int FPS = 60;                 // FPS
    private final int THV = 250;                // Threshold

    private volatile boolean running = true;   // Steuerung des Tracker-Threads

    private volatile Point currentPositionP1 = new Point(0, 0);
    private volatile Point currentPositionP2 = new Point(0, 0);

    public Tracker() {
    }

    public static void main(String[] args) {
        Tracker tracker = new Tracker();
        Thread trackerThread = new Thread(tracker);
        trackerThread.start();

        // Beispiel: Positionen regelmäßig abfragen
        /*
        while (true) {
            Point p1 = tracker.getP1();
            Point p2 = tracker.getP2();
            //System.out.println("P2 - X: " + p2.x + " Y: " + p2.y);

            try {
                Thread.sleep(1000); // Warte 1 Sekunde
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
         */
    }

    @Override
    public void run() {
        VideoCapture capture = setUp();

        if (!capture.isOpened()) {
            System.out.println("Fehler: Die Webcam konnte nicht geöffnet werden.");
            return;
        }

        Mat frame = new Mat();
        Mat grayFrame = new Mat();
        Mat thresholdFrame = new Mat();

        while (running) {
            if (capture.read(frame)) {

                // Konvertiere in Graustufen und wende Threshold an
                Imgproc.cvtColor(frame, grayFrame, Imgproc.COLOR_BGR2GRAY);
                Imgproc.threshold(grayFrame, thresholdFrame, THV, 255, Imgproc.THRESH_BINARY);

                // Tracke die Positionen der beiden größten Objekte
                List<Point> positions = trackPositions(thresholdFrame);
                //Check

                if (positions.size() >= 2) {
                    setCurrentPositionP1(positions.get(0));
                    //System.out.println(currentPositionP1);
                    //Check. currentPosition wird aktualisiert
                    setCurrentPositionP2(positions.get(1));
                    //Check

                    //System.out.println("P1 - X: " + currentPositionP1.x + " Y: " + currentPositionP1.y +
                    //        " P2 - X: " + currentPositionP2.x + " Y: " + currentPositionP2.y);
                }

                HighGui.imshow("GTCar", frame);

                if (HighGui.waitKey(1) == 27) { // ESC-Taste zum Beenden
                    stop();
                }
            } else {
                System.out.println("Fehler: Das Kamerabild konnte nicht gelesen werden.");
                break;
            }
        }
        capture.release();
    }

    private List<Point> trackPositions(Mat thresholdFrame) {
        List<MatOfPoint> contours = new ArrayList<>();
        Mat hierarchy = new Mat();

        // Finde Konturen
        Imgproc.findContours(thresholdFrame, contours, hierarchy, Imgproc.RETR_EXTERNAL, Imgproc.CHAIN_APPROX_SIMPLE);

        // Sortiere Konturen nach Fläche (absteigend)
        contours.sort((c1, c2) -> Double.compare(Imgproc.contourArea(c2), Imgproc.contourArea(c1)));

        // Speichere die Positionen der zwei größten Konturen
        List<Point> positions = new ArrayList<>();

        for (int i = 0; i < Math.min(contours.size(), 2); i++) {
            Moments moments = Imgproc.moments(contours.get(i));
            if (moments.get_m00() != 0) {
                int cx = (int) (moments.get_m10() / moments.get_m00());
                int cy = (int) (moments.get_m01() / moments.get_m00());
                positions.add(new Point(cx, cy));
            }
        }
        //Check
        return positions;
    }

    public synchronized Point getP1() {
        //ab hier ist Punkt[0,0]
        return currentPositionP1;
    }

    public synchronized Point getP2() {
        return currentPositionP2;
    }

    private synchronized void setCurrentPositionP1(Point p1) {
        //p1 ist richtig
        currentPositionP1 = p1;
    }

    private synchronized void setCurrentPositionP2(Point p2) {
        currentPositionP2 = p2;
    }

    private VideoCapture setUp() {
        System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
        VideoCapture capture = new VideoCapture(CAM);

        if (!capture.isOpened()) {
            System.out.println("Fehler: Die Webcam konnte nicht geöffnet werden.");
        }

        capture.set(Videoio.CAP_PROP_EXPOSURE, EXP);
        capture.set(Videoio.CAP_PROP_FPS, FPS);

        return capture;
    }

    public void stop() {
        running = false;
    }
}
